@@ -17,9 +17,16 @@ import enum
 import json
 import logging
 import os
+import select
+import sys
+import time
+
+import pyudev
 
 from subiquitycore.controller import BaseController
+from subiquitycore.utils import run_command
 
+from subiquity.controllers.error import ErrorReportKind
 from subiquity.models.filesystem import (
     align_up,
     Bootloader,
@@ -49,11 +56,78 @@ UEFI_GRUB_SIZE_BYTES = 512 * 1024 * 1024  # 512MiB EFI partition
 class ProbeState(enum.IntEnum):
     NOT_STARTED = enum.auto()
     PROBING = enum.auto()
-    REPROBING = enum.auto()
     FAILED = enum.auto()
     DONE = enum.auto()
 
 
+class Probe:
+
+    def __init__(self, controller, restricted, timeout, cb):
+        self.controller = controller
+        self.restricted = restricted
+        self.timeout = timeout
+        self.cb = cb
+        self.state = ProbeState.NOT_STARTED
+        self.result = None
+        if restricted:
+            self.kind = ErrorReportKind.DISK_PROBE_FAIL
+        else:
+            self.kind = ErrorReportKind.BLOCK_PROBE_FAIL
+        self.crash_report = None
+
+    def start(self):
+        block_discover_log.debug(
+            "starting probe restricted=%s", self.restricted)
+        self.state = ProbeState.PROBING
+        self.controller.run_in_bg(self._bg_probe, self._probed)
+        self.controller.loop.set_alarm_in(self.timeout, self._check_timeout)
+
+    def _bg_probe(self):
+        if self.restricted:
+            probe_types = {'blockdev'}
+        else:
+            probe_types = None
+        debug_flags = self.controller.debug_flags
+        if 'bpfail-full' in debug_flags and not self.restricted:
+            time.sleep(2)
+            1/0
+        if 'bpfail-restricted' in debug_flags and self.restricted:
+            time.sleep(2)
+            1/0
+        # Should consider invoking probert in a subprocess here (so we
+        # can kill it if it gets stuck).
+        return self.controller.app.prober.get_storage(probe_types=probe_types)
+
+    def _probed(self, fut):
+        if self.state == ProbeState.FAILED:
+            block_discover_log.debug(
+                "ignoring result %s for timed out probe", fut)
+            return
+        try:
+            self.result = fut.result()
+        except Exception:
+            block_discover_log.exception(
+                "probing failed restricted=%s", self.restricted)
+            self.crash_report = self.controller.app.make_apport_report(
+                self.kind, "block probing", interrupt=False)
+            self.state = ProbeState.FAILED
+        else:
+            block_discover_log.info(
+                "probing successful restricted=%s", self.restricted)
+            self.state = ProbeState.DONE
+        self.cb(self)
+
+    def _check_timeout(self, loop, ud):
+        if self.state != ProbeState.PROBING:
+            return
+        self.crash_report = self.controller.app.make_apport_report(
+            self.kind, "block probing timed out", interrupt=False)
+        block_discover_log.exception(
+            "probing timed out restricted=%s", self.restricted)
+        self.state = ProbeState.FAILED
+        self.cb(self)
+
+
 class FilesystemController(BaseController):
 
     def __init__(self, app):
@@ -65,73 +139,127 @@ class FilesystemController(BaseController):
         self.answers.setdefault('guided', False)
         self.answers.setdefault('guided-index', 0)
         self.answers.setdefault('manual', [])
-        self._probe_state = ProbeState.NOT_STARTED
+        self._cur_probe = None
+        self._monitor = None
+        self._udev_listen_handle = None
+        self._probes = {}
 
     def start(self):
-        block_discover_log.info("starting probe")
-        self._probe_state = ProbeState.PROBING
-        self.run_in_bg(self._bg_probe, self._probed)
-        self.loop.set_alarm_in(
-            5.0, lambda loop, ud: self._check_probe_timeout())
+        self._start_probe(restricted=False, start=False)
+        target = self.app.base_model.target
+        if os.path.exists(target):
+            self.run_in_bg(self._bg_unmount, self._unmounted)
+        else:
+            self._cur_probe.start()
 
-    def _bg_probe(self, probe_types=None):
-        return self.app.prober.get_storage(probe_types=probe_types)
+    def _bg_unmount(self):
+        cmd = [
+            sys.executable, '-m', 'curtin', 'unmount',
+            '-t', self.app.base_model.target,
+            ]
+        if self.opts.dry_run:
+            cmd = ['sleep', 0.2]
+        run_command(cmd)
 
-    def _probed(self, fut, restricted=False):
-        if not restricted and self._probe_state != ProbeState.PROBING:
-            block_discover_log.debug(
-                "ignoring result %s for timed out probe", fut)
+    def _unmounted(self, fut):
+        context = pyudev.Context()
+        self._monitor = pyudev.Monitor.from_netlink(context)
+        self._monitor.filter_by(subsystem='block')
+        self._monitor.enable_receiving()
+        self.start_listening_udev()
+        self._cur_probe.start()
+
+    def start_listening_udev(self):
+        self._udev_listen_handle = self.loop.watch_file(
+            self._monitor.fileno(), self._udev_event)
+
+    def stop_listening_udev(self):
+        if self._udev_listen_handle is not None:
+            self.loop.remove_watch_file(self._udev_listen_handle)
+            self._udev_listen_handle = None
+
+    def _udev_event(self):
+        cp = run_command(['udevadm', 'settle', '-t', '0'])
+        if cp.returncode != 0:
+            log.debug("waiting 0.1 to let udev event queue settle")
+            self.stop_listening_udev()
+            self.loop.set_alarm_in(
+                0.1, lambda loop, ud: self.start_listening_udev())
             return
-        try:
-            storage = fut.result()
-            if restricted:
-                fname = 'probe-data-restricted.json'
+        # Drain the udev events in the queue -- if we stopped listening to
+        # allow udev to settle, it's good bet there is more than one event to
+        # process and we don't want to kick off a full block probe for each
+        # one.  It's a touch unfortunate that pyudev doesn't have a
+        # non-blocking read so we resort to select().
+        while select.select([self._monitor.fileno()], [], [], 0)[0]:
+            action, dev = self._monitor.receive_device()
+            log.debug("_udev_event %s %s", action, dev)
+        self._start_probe(restricted=False)
+
+    def _start_probe(self, *, restricted, start=True):
+        p = Probe(self, restricted, 5.0, self._probe_done)
+        self._cur_probe = self._probes[restricted] = p
+        if start:
+            p.start()
+
+    def _probe_done(self, probe):
+        if probe is not self._cur_probe:
+            block_discover_log.debug(
+                "ignoring result %s for superseded probe", probe.result)
+            return
+        if probe.state == ProbeState.FAILED:
+            if not probe.restricted:
+                self._start_probe(restricted=True)
             else:
-                fname = 'probe-data.json'
-            with open(os.path.join(self.app.block_log_dir, fname), 'w') as fp:
-                json.dump(storage, fp, indent=4)
-            self.model.load_probe_data(storage)
+                if self.showing:
+                    self.start_ui()
+            return
+        if probe.restricted:
+            fname = 'probe-data-restricted.json'
+            key = "ProbeDataRestricted"
+        else:
+            fname = 'probe-data.json'
+            key = "ProbeData"
+        fpath = os.path.join(self.app.block_log_dir, fname)
+        with open(fpath, 'w') as fp:
+            json.dump(probe.result, fp, indent=4)
+        self.app.note_file_for_apport(key, fpath)
+        try:
+            self.model.load_probe_data(probe.result)
         except Exception:
             block_discover_log.exception(
-                "probing failed restricted=%s", restricted)
-            if not restricted:
-                block_discover_log.info("reprobing for blockdev only")
-                # Should make a crash file for apport, arrange for it to be
-                # copied onto the installed system and tell user all this
-                # happened!
-                self._reprobe()
+                "load_probe_data failed restricted=%s", probe.restricted)
+            probe.crash_report = self.app.make_apport_report(
+                probe.kind, "loading probe data", interrupt=False)
+            if not probe.restricted:
+                self._start_probe(restricted=True)
             else:
-                self._probe_state = ProbeState.FAILED
+                # OK, this is a hack
+                self._cur_probe.state = ProbeState.FAILED
                 if self.showing:
-                    self.default()
+                    self.start_ui()
         else:
-            self._probe_state = ProbeState.DONE
             # Should do something here if probing found no devices.
             if self.showing:
-                self.default()
-
-    def _check_probe_timeout(self):
-        log.debug("_check_probe_timeout")
-        if self._probe_state == ProbeState.PROBING:
-            log.info(
-                "unrestricted probing timed out, reprobing for blockdev only")
-            self._reprobe()
-
-    def _reprobe(self):
-        self._probe_state = ProbeState.REPROBING
-        self.run_in_bg(
-            lambda: self._bg_probe({"blockdev"}),
-            lambda fut: self._probed(fut, True),
-            )
+                self.start_ui()
 
     def start_ui(self):
-        if self._probe_state in [ProbeState.PROBING,
-                                 ProbeState.REPROBING]:
+        if self._cur_probe.state == ProbeState.PROBING:
             self.ui.set_body(SlowProbing(self))
-        elif self._probe_state == ProbeState.FAILED:
+        elif self._cur_probe.state == ProbeState.FAILED:
             self.ui.set_body(ProbingFailed(self))
+            self.ui.body.show_error()
         else:
+            # Once we've shown the filesystem UI, we stop listening for udev
+            # events as merging system changes with configuration the user has
+            # performed would be tricky.  Possibly worth doing though! Just
+            # not today.
+            self.stop_listening_udev()
             self.ui.set_body(GuidedFilesystemView(self))
+            if self._cur_probe.restricted:
+                pr = self._probes[False].crash_report
+                if pr is not None:
+                    self.app.show_error_report(pr)
             if self.answers['guided']:
                 self.guided(self.answers.get('guided-method', 'direct'))
             elif self.answers['manual']:
@@ -319,13 +447,13 @@ class FilesystemController(BaseController):
             part_size = UEFI_GRUB_SIZE_BYTES
             if UEFI_GRUB_SIZE_BYTES*2 >= disk.size:
                 part_size = disk.size // 2
-            log.debug('Adding EFI partition first')
+            log.debug('_create_boot_partition - adding EFI partition')
             part = self.create_partition(
                 disk,
                 dict(size=part_size, fstype='fat32', mount='/boot/efi'),
                 flag="boot")
         elif bootloader == Bootloader.PREP:
-            log.debug('Adding PReP gpt partition first')
+            log.debug('_create_boot_partition - adding PReP partition')
             part = self.create_partition(
                 disk,
                 dict(size=PREP_GRUB_SIZE_BYTES, fstype=None, mount=None),
@@ -334,7 +462,7 @@ class FilesystemController(BaseController):
                 flag='prep')
             self.model.grub_install_device = part
         elif bootloader == Bootloader.BIOS:
-            log.debug('Adding grub_bios gpt partition first')
+            log.debug('_create_boot_partition - adding bios_grub partition')
             part = self.create_partition(
                 disk,
                 dict(size=BIOS_GRUB_SIZE_BYTES, fstype=None, mount=None),
@@ -445,7 +573,7 @@ class FilesystemController(BaseController):
 
         self.create_partition(disk, spec)
 
-        log.info("Successfully added partition")
+        log.debug("Successfully added partition")
 
     def logical_volume_handler(self, vg, lv, spec):
         log.debug('logical_volume_handler: %s %s %s', vg, lv, spec)
