@@ -18,6 +18,7 @@ from concurrent.futures import Future
 import datetime
 import logging
 import os
+import re
 import signal
 import subprocess
 import sys
@@ -26,14 +27,21 @@ import tempfile
 import time
 import traceback
 
+from curtin.commands.install import (
+    ERROR_TARFILE,
+    INSTALL_LOG,
+    )
+
 import urwid
-import yaml
 
 from systemd import journal
 
+import yaml
+
 from subiquitycore import utils
 from subiquitycore.controller import BaseController
 
+from subiquity.controllers.error import ErrorReportKind
 from subiquity.ui.views.installprogress import ProgressView
 
 
@@ -155,7 +163,7 @@ class StateMachine:
             raise
         except Exception:
             log.debug("%s failed", name)
-            self.controller.curtin_error(traceback.format_exc())
+            self.controller.curtin_error()
         else:
             log.debug("%s completed", name)
             if 'success' in self._transitions[name]:
@@ -198,6 +206,25 @@ class StateMachine:
         self.run()
 
 
+class TracebackExtractor:
+
+    start_marker = re.compile(r"^Traceback \(most recent call last\):")
+    end_marker = re.compile(r"\S")
+
+    def __init__(self):
+        self.traceback = []
+        self.in_traceback = False
+
+    def feed(self, line):
+        if not self.traceback and self.start_marker.match(line):
+            self.in_traceback = True
+        elif self.in_traceback and self.end_marker.match(line):
+            self.traceback.append(line)
+            self.in_traceback = False
+        if self.in_traceback:
+            self.traceback.append(line)
+
+
 class InstallProgressController(BaseController):
     signals = [
         ('installprogress:filesystem-config-done', 'filesystem_config_done'),
@@ -223,6 +250,7 @@ class InstallProgressController(BaseController):
         self._event_syslog_identifier = 'curtin_event.%s' % (os.getpid(),)
         self._log_syslog_identifier = 'curtin_log.%s' % (os.getpid(),)
         self.sm = None
+        self.tb_extractor = TracebackExtractor()
 
     def tpath(self, *path):
         return os.path.join(self.model.target, *path)
@@ -245,16 +273,21 @@ class InstallProgressController(BaseController):
     def snap_config_done(self):
         self._step_done('snap')
 
-    def curtin_error(self, log_text=None):
-        log.debug('curtin_error: %s', log_text)
+    def curtin_error(self):
         self.install_state = InstallState.ERROR
+        kw = {}
+        if self.tb_extractor.traceback:
+            kw["Traceback"] = "\n".join(self.tb_extractor.traceback)
+        crash_report = self.app.make_apport_report(
+            ErrorReportKind.INSTALL_FAIL, "install failed", interrupt=False,
+            **kw)
         self.progress_view.spinner.stop()
-        if log_text:
-            self.progress_view.add_log_line(log_text)
+        if sys.exc_info()[0] is not None:
+            self.progress_view.add_log_line(traceback.format_exc())
         self.progress_view.set_status(('info_error',
                                        _("An error has occurred")))
-        self.progress_view.show_complete(True)
-        self.default()
+        self.start_ui()
+        self.progress_view.show_error(crash_report)
 
     def _bg_run_command_logged(self, cmd, **kwargs):
         cmd = ['systemd-cat', '--level-prefix=false',
@@ -269,15 +302,14 @@ class InstallProgressController(BaseController):
 
     def _install_event_start(self, message):
         log.debug("_install_event_start %s", message)
-        self.footer_description.set_text(message)
         self.progress_view.add_event(self._event_indent + message)
         self._event_indent += "  "
-        self.footer_spinner.start()
+        self.progress_view.spinner.start()
 
     def _install_event_finish(self):
         self._event_indent = self._event_indent[:-2]
         log.debug("_install_event_finish %r", self._event_indent)
-        self.footer_spinner.stop()
+        self.progress_view.spinner.stop()
 
     def curtin_event(self, event):
         e = {}
@@ -294,7 +326,9 @@ class InstallProgressController(BaseController):
             self._install_event_finish()
 
     def curtin_log(self, event):
-        self.progress_view.add_log_line(event['MESSAGE'])
+        log_line = event['MESSAGE']
+        self.progress_view.add_log_line(log_line)
+        self.tb_extractor.feed(log_line)
 
     def start_journald_listener(self, identifiers, callback):
         reader = journal.Reader()
@@ -321,36 +355,36 @@ class InstallProgressController(BaseController):
         config_file_name = 'subiquity-curtin-install.conf'
 
         if self.opts.dry_run:
-            log.debug("Installprogress: this is a dry-run")
             config_location = os.path.join('.subiquity/', config_file_name)
-            curtin_cmd = ["python3", "scripts/replay-curtin-log.py",
-                          "examples/curtin-events.json",
-                          self._event_syslog_identifier]
+            log_location = '.subiquity/install.log'
+            event_file = "examples/curtin-events.json"
+            if 'install-fail' in self.debug_flags:
+                event_file = "examples/curtin-events-fail.json"
+            curtin_cmd = [
+                "python3", "scripts/replay-curtin-log.py", event_file,
+                self._event_syslog_identifier, log_location,
+                ]
         else:
-            log.debug("Installprogress: this is the *REAL* thing")
             config_location = os.path.join('/var/log/installer',
                                            config_file_name)
             curtin_cmd = [sys.executable, '-m', 'curtin', '--showtrace', '-c',
                           config_location, 'install']
+            log_location = INSTALL_LOG
 
         ident = self._event_syslog_identifier
         self._write_config(config_location,
                            self.model.render(syslog_identifier=ident))
 
+        self.app.note_file_for_apport("CurtinConfig", config_location)
+        self.app.note_file_for_apport("CurtinLog", log_location)
+        self.app.note_file_for_apport("CurtinErrors", ERROR_TARFILE)
+
         return curtin_cmd
 
     def curtin_start_install(self):
-        log.debug('Curtin Install: starting curtin')
+        log.debug('curtin_start_install')
         self.install_state = InstallState.RUNNING
-        self.footer_description = urwid.Text(_("starting..."))
         self.progress_view = ProgressView(self)
-        self.footer_spinner = self.progress_view.spinner
-
-        self.ui.auto_footer = False
-        self.ui.set_footer(urwid.Columns(
-            [('pack', urwid.Text(_("Install in progress:"))),
-             (self.footer_description),
-             ('pack', self.footer_spinner)], dividechars=1))
 
         self.journal_listener_handle = self.start_journald_listener(
             [self._event_syslog_identifier, self._log_syslog_identifier],
@@ -358,7 +392,7 @@ class InstallProgressController(BaseController):
 
         curtin_cmd = self._get_curtin_command()
 
-        log.debug('Curtin install cmd: {}'.format(curtin_cmd))
+        log.debug('curtin install cmd: {}'.format(curtin_cmd))
         self.run_in_bg(
             lambda: self._bg_run_command_logged(curtin_cmd),
             self.curtin_install_completed)
@@ -371,12 +405,6 @@ class InstallProgressController(BaseController):
             return
         self.install_state = InstallState.DONE
         log.debug('After curtin install OK')
-        self.ui.progress_current += 1
-        if not self.showing:
-            self.ui.set_footer(_("Install complete"))
-        else:
-            # Re-set footer so progress bar updates.
-            self.ui.set_footer(_("Thank you for using Ubuntu!"))
         self._step_done('install')
 
     def cancel(self):
@@ -515,6 +543,8 @@ class InstallProgressController(BaseController):
           transitions={'reboot': 'reboot'})
     def _bg_copy_logs_to_target(self):
         if self.opts.dry_run:
+            if 'copy-logs-fail' in self.debug_flags:
+                raise PermissionError()
             return
         target_logs = self.tpath('var/log/installer')
         utils.run_command(['cp', '-aT', '/var/log/installer', target_logs])
@@ -566,16 +596,12 @@ class InstallProgressController(BaseController):
     def start_ui(self):
         if self.install_state == InstallState.RUNNING:
             self.progress_view.title = _("Installing system")
-            footer = _("Thank you for using Ubuntu!")
         elif self.install_state == InstallState.DONE:
             self.progress_view.title = _("Install complete!")
-            footer = _("Thank you for using Ubuntu!")
         elif self.install_state == InstallState.ERROR:
             self.progress_view.title = (
                 _('An error occurred during installation'))
-            footer = _('Please report this error in Launchpad')
         self.ui.set_body(self.progress_view)
-        self.ui.set_footer(footer)
 
 
 uu_apt_conf = """\
