@@ -183,7 +183,7 @@ class AutoDetectPressKey(AutoDetectBase):
             ])
 
     def keypress(self, size, key):
-        log.debug('keypress %r %r', size, key)
+        log.debug('keypress %r', key)
         if key.startswith('release '):
             # Escape is key 1 on all keyboards and all layouts except
             # amigas and very old Macs so this seems safe enough.
@@ -385,7 +385,6 @@ class KeyboardForm(Form):
 class KeyboardView(BaseView):
 
     title = _("Keyboard configuration")
-    footer = _("Use UP, DOWN and ENTER keys to select your keyboard.")
 
     def __init__(self, model, controller, opts):
         self.model = model
@@ -450,7 +449,7 @@ class KeyboardView(BaseView):
         self.form.layout.widget.value = layout
         self.select_layout(None, layout)
         self.form.variant.widget.value = variant
-        self._w.base_widget.focus_position = 5
+        self._w.base_widget.focus_position = 4
 
     def done(self, result):
         layout = self.form.layout.widget.value
@@ -474,7 +473,8 @@ class KeyboardView(BaseView):
         self.controller.cancel()
 
     def select_layout(self, sender, layout):
-        log.debug("%s", layout)
+        if sender is not None:
+            log.debug("select_layout %s", layout)
         opts = []
         default_i = -1
         layout_items = enumerate(self.model.variants[layout].items())
