@@ -28,17 +28,12 @@ import yaml
 
 from subiquitycore.controller import RepeatedController
 from subiquitycore.signals import Signal
-from subiquitycore.prober import Prober, ProberException
+from subiquitycore.prober import Prober
 from subiquitycore.ui.frame import SubiquityCoreUI
 
 log = logging.getLogger('subiquitycore.core')
 
 
-class ApplicationError(Exception):
-    """ Basecontroller exception """
-    pass
-
-
 class Skip(Exception):
     """Raise this from a controller's default method to skip a screen."""
 
@@ -61,20 +56,36 @@ KDGKBMODE = 0x4B44  # gets current keyboard mode
 KDSKBMODE = 0x4B45  # sets current keyboard mode
 
 
-class ISO_8613_3_Screen(urwid.raw_display.Screen):
+class TwentyFourBitScreen(urwid.raw_display.Screen):
 
     def __init__(self, _urwid_name_to_rgb):
-        self._fg_to_rgb = _urwid_name_to_rgb.copy()
-        self._fg_to_rgb['default'] = _urwid_name_to_rgb['light gray']
-        self._bg_to_rgb = _urwid_name_to_rgb.copy()
-        self._bg_to_rgb['default'] = _urwid_name_to_rgb['black']
+        self._urwid_name_to_rgb = _urwid_name_to_rgb
         super().__init__()
 
+    def _cc(self, color):
+        """Return the "SGR" parameter for selecting color.
+
+        See https://en.wikipedia.org/wiki/ANSI_escape_code#SGR for an
+        explanation.  We use the basic codes for black/white/default for
+        maximum compatibility; they are the only colors used when the
+        mono palette is selected.
+        """
+        if color == 'white':
+            return '7'
+        elif color == 'black':
+            return '0'
+        elif color == 'default':
+            return '9'
+        else:
+            # This is almost but not quite a ISO 8613-3 code -- that
+            # would use colons to separate the rgb values instead. But
+            # it's what xterm, and hence everything else, supports.
+            return '8;2;{};{};{}'.format(*self._urwid_name_to_rgb[color])
+
     def _attrspec_to_escape(self, a):
-        f_r, f_g, f_b = self._fg_to_rgb[a.foreground]
-        b_r, b_g, b_b = self._bg_to_rgb[a.background]
-        return "\x1b[38;2;{};{};{};48;2;{};{};{}m".format(f_r, f_g, f_b,
-                                                          b_r, b_g, b_b)
+        return '\x1b[0;3{};4{}m'.format(
+            self._cc(a.foreground),
+            self._cc(a.background))
 
 
 def is_linux_tty():
@@ -83,12 +94,24 @@ def is_linux_tty():
     except IOError as e:
         log.debug("KDGKBTYPE failed %r", e)
         return False
-    log.debug("KDGKBTYPE returned %r", r)
+    log.debug("KDGKBTYPE returned %r, is_linux_tty %s", r, r == b'\x02')
     return r == b'\x02'
 
 
-def setup_screen(colors, styles):
-    """Return a palette and screen to be passed to MainLoop.
+urwid_8_names = (
+    'black',
+    'dark red',
+    'dark green',
+    'brown',
+    'dark blue',
+    'dark magenta',
+    'dark cyan',
+    'light gray',
+)
+
+
+def make_palette(colors, styles, ascii):
+    """Return a palette to be passed to MainLoop.
 
     colors is a list of exactly 8 tuples (name, (r, g, b))
 
@@ -99,42 +122,74 @@ def setup_screen(colors, styles):
     # to the basic colors by their "standard" names but we overwrite
     # these colors to mean different things.  So we convert styles into
     # an urwid palette by mapping the names in colors to the standard
-    # name, and then either overwrite the first 8 colors to be the
-    # colors from 'colors' (on the linux vt) or use a custom screen
-    # class that displays maps the standard color name to the value
-    # specified in colors using 24-bit control codes.
+    # name.
     if len(colors) != 8:
         raise Exception(
-            "setup_screen must be passed a list of exactly 8 colors")
-    urwid_8_names = (
-        'black',
-        'dark red',
-        'dark green',
-        'brown',
-        'dark blue',
-        'dark magenta',
-        'dark cyan',
-        'light gray',
-    )
+            "make_palette must be passed a list of exactly 8 colors")
     urwid_name = dict(zip([c[0] for c in colors], urwid_8_names))
 
     urwid_palette = []
     for name, fg, bg in styles:
-        urwid_palette.append((name, urwid_name[fg], urwid_name[bg]))
+        urwid_fg, urwid_bg = urwid_name[fg], urwid_name[bg]
+        if ascii:
+            # 24bit grey on colored background looks good
+            # but in 16 colors it's unreadable
+            # hence add more contrast
+            if urwid_bg != 'black':
+                urwid_fg = 'black'
+            # Only frame_button doesn't match above rule
+            # fix it to be brown-on-black black-on-brown
+            if name == 'frame_button focus':
+                urwid_fg, urwid_bg = 'brown', 'black'
+        urwid_palette.append((name, urwid_fg, urwid_bg))
 
-    if is_linux_tty():
+    return urwid_palette
+
+
+def make_screen(colors, is_linux_tty, ascii_mode):
+    """Return a screen to be passed to MainLoop.
+
+    colors is a list of exactly 8 tuples (name, (r, g, b)), the same as
+    passed to make_palette.
+    """
+    # On the linux console, we overwrite the first 8 colors to be those
+    # defined by colors. Otherwise, we return a screen that uses ISO
+    # 8613-3ish codes to display the colors.
+    if len(colors) != 8:
+        raise Exception(
+            "make_screen must be passed a list of exactly 8 colors")
+    if is_linux_tty:
+        # Perhaps we ought to return a screen subclass that does this
+        # ioctl-ing in .start() and undoes it in .stop() but well.
         curpal = bytearray(16*3)
         fcntl.ioctl(sys.stdout.fileno(), GIO_CMAP, curpal)
         for i in range(8):
             for j in range(3):
                 curpal[i*3+j] = colors[i][1][j]
         fcntl.ioctl(sys.stdout.fileno(), PIO_CMAP, curpal)
-        return urwid.raw_display.Screen(), urwid_palette
+        return urwid.raw_display.Screen()
+    elif ascii_mode:
+        return urwid.raw_display.Screen()
     else:
         _urwid_name_to_rgb = {}
         for i, n in enumerate(urwid_8_names):
             _urwid_name_to_rgb[n] = colors[i][1]
-        return ISO_8613_3_Screen(_urwid_name_to_rgb), urwid_palette
+        return TwentyFourBitScreen(_urwid_name_to_rgb)
+
+
+def extend_dec_special_charmap():
+    urwid.escape.DEC_SPECIAL_CHARMAP.update({
+        ord('\N{BLACK RIGHT-POINTING SMALL TRIANGLE}'): '>',
+        ord('\N{BLACK LEFT-POINTING SMALL TRIANGLE}'): '<',
+        ord('\N{BLACK DOWN-POINTING SMALL TRIANGLE}'): 'v',
+        ord('\N{BLACK UP-POINTING SMALL TRIANGLE}'): '^',
+        ord('\N{check mark}'): '+',
+        ord('\N{bullet}'): '*',
+        ord('\N{lower half block}'): '=',
+        ord('\N{upper half block}'): '=',
+        ord('\N{FULL BLOCK}'): urwid.escape.DEC_SPECIAL_CHARMAP[
+            ord('\N{BOX DRAWINGS LIGHT VERTICAL}')],
+    })
 
 
 class KeyCodesFilter:
@@ -164,14 +219,11 @@ class KeyCodesFilter:
         self._old_mode = struct.unpack('i', o)[0]
         # Set the keyboard mode to K_MEDIUMRAW, which causes the keyboard
         # driver in the kernel to pass us keycodes.
-        log.debug("old mode was %s, setting mode to %s",
-                  self._old_mode, K_MEDIUMRAW)
         fcntl.ioctl(self._fd, KDSKBMODE, K_MEDIUMRAW)
 
     def exit_keycodes_mode(self):
         log.debug("exit_keycodes_mode")
         self.filtering = False
-        log.debug("setting mode back to %s", self._old_mode)
         fcntl.ioctl(self._fd, KDSKBMODE, self._old_mode)
 
     def filter(self, keys, codes):
@@ -216,6 +268,59 @@ class DummyKeycodesFilter:
         return keys
 
 
+class AsyncioEventLoop(urwid.AsyncioEventLoop):
+    # This is fixed in the latest urwid.
+
+    def _exception_handler(self, loop, context):
+        exc = context.get('exception')
+        if exc:
+            loop.stop()
+            if not isinstance(exc, urwid.ExitMainLoop):
+                # Store the exc_info so we can re-raise after the loop stops
+                self._exc_info = (type(exc), exc, exc.__traceback__)
+        else:
+            loop.default_exception_handler(context)
+
+
+class ControllerSet:
+
+    def __init__(self, app, names):
+        self.app = app
+        self.controller_names = names
+        self.index = -1
+        self.instances = []
+
+    def load(self):
+        controllers_mod = __import__(
+            '{}.controllers'.format(self.app.project), None, None, [''])
+        for name in self.controller_names:
+            log.debug("Importing controller: %s", name)
+            klass = getattr(controllers_mod, name+"Controller")
+            if hasattr(self, name):
+                c = 0
+                for instance in self.instances:
+                    if isinstance(instance, klass):
+                        c += 1
+                inst = RepeatedController(getattr(self, name), c)
+                name = inst.name
+            else:
+                inst = klass(self.app)
+            setattr(self, name, inst)
+            self.instances.append(inst)
+
+    @property
+    def cur(self):
+        if self.out_of_bounds():
+            return None
+        inst = self.instances[self.index]
+        while isinstance(inst, RepeatedController):
+            inst = inst.orig
+        return inst
+
+    def out_of_bounds(self):
+        return self.index < 0 or self.index >= len(self.instances)
+
+
 class Application:
 
     # A concrete subclass must set project and controllers attributes, e.g.:
@@ -229,18 +334,13 @@ class Application:
     #         "InstallProgress",
     # ]
     # The 'next-screen' and 'prev-screen' signals move through the list of
-    # controllers in order, calling the default method on the controller
+    # controllers in order, calling the start_ui method on the controller
     # instance.
 
     make_ui = SubiquityCoreUI
 
     def __init__(self, opts):
-        try:
-            prober = Prober(opts)
-        except ProberException as e:
-            err = "Prober init failed: {}".format(e)
-            log.exception(err)
-            raise ApplicationError(err)
+        prober = Prober(opts)
 
         self.ui = self.make_ui()
         self.opts = opts
@@ -259,8 +359,12 @@ class Application:
             if not opts.dry_run:
                 open('/run/casper-no-prompt', 'w').close()
 
-        if is_linux_tty():
-            log.debug("is_linux_tty")
+        self.is_color = False
+        self.color_palette = make_palette(self.COLORS, self.STYLES, opts.ascii)
+
+        self.is_linux_tty = is_linux_tty()
+
+        if self.is_linux_tty:
             self.input_filter = KeyCodesFilter()
         else:
             self.input_filter = DummyKeycodesFilter()
@@ -272,21 +376,7 @@ class Application:
         self.prober = prober
         self.loop = None
         self.pool = futures.ThreadPoolExecutor(10)
-        if opts.screens:
-            self.controllers = [c for c in self.controllers
-                                if c in opts.screens]
-        else:
-            self.controllers = self.controllers[:]
-        self.ui.progress_completion = len(self.controllers)
-        self.controller_instances = dict.fromkeys(self.controllers)
-        self.controller_index = -1
-
-    @property
-    def cur_controller(self):
-        if self.controller_index < 0:
-            return None
-        controller_name = self.controllers[self.controller_index]
-        return self.controller_instances[controller_name]
+        self.controllers = ControllerSet(self, self.controllers)
 
     def run_in_bg(self, func, callback):
         """Run func() in a thread and call callback on UI thread.
@@ -308,104 +398,110 @@ class Application:
             os.write(pipe, b'x')
         fut.add_done_callback(in_random_thread)
 
-    def run_command_in_foreground(self, cmd, **kw):
+    def run_command_in_foreground(self, cmd, before_hook=None, after_hook=None,
+                                  **kw):
         screen = self.loop.screen
 
+        # Calling screen.stop() sends the INPUT_DESCRIPTORS_CHANGED
+        # signal. This calls _reset_input_descriptors() which calls
+        # unhook_event_loop / hook_event_loop on the screen. But this all
+        # happens before _started is set to False on the screen and so this
+        # does not actually do anything -- we end up attempting to read from
+        # stdin while in a background process group, something that gets the
+        # kernel upset at us.
+        #
+        # The cleanest fix seems to be to just send the signal again once
+        # stop() has returned which, now that screen._started is False,
+        # correctly stops listening from stdin.
+        #
+        # There is an exactly analagous problem with screen.start() except
+        # there the symptom is that we are running in the foreground but not
+        # listening to stdin! The fix is the same.
+
         def run():
             subprocess.run(cmd, **kw)
 
         def restore(fut):
             screen.start()
-            # Calling screen.start() sends the INPUT_DESCRIPTORS_CHANGED
-            # signal. This calls _reset_input_descriptors() which calls
-            # unhook_event_loop / hook_event_loop on the screen. But this all
-            # happens before _started is set on the screen, so hook_event_loop
-            # does not actually do anything -- and we end up not listening to
-            # stdin, obviously a defective situation for a console
-            # application. So send it again now the screen is started...
             urwid.emit_signal(
                 screen, urwid.display_common.INPUT_DESCRIPTORS_CHANGED)
             tty.setraw(0)
+            if after_hook is not None:
+                after_hook()
 
         screen.stop()
+        urwid.emit_signal(
+            screen, urwid.display_common.INPUT_DESCRIPTORS_CHANGED)
+        if before_hook is not None:
+            before_hook()
         self.run_in_bg(run, restore)
 
     def _connect_base_signals(self):
-        """ Connect signals used in the core controller
-        """
-        signals = []
-
-        signals.append(('quit', self.exit))
+        """Connect signals used in the core controller."""
+        signals = [
+            ('quit', self.exit),
+            ('next-screen', self.next_screen),
+            ('prev-screen', self.prev_screen),
+            ]
         if self.opts.dry_run:
             signals.append(('control-x-quit', self.exit))
-        signals.append(('refresh', self.redraw_screen))
-        signals.append(('next-screen', self.next_screen))
-        signals.append(('prev-screen', self.prev_screen))
         self.signal.connect_signals(signals)
 
         # Registers signals from each controller
-        for controller_class in self.controller_instances.values():
-            controller_class.register_signals()
-        log.debug(self.signal)
+        for controller in self.controllers.instances:
+            controller.register_signals()
+        log.debug("known signals: %s", self.signal.known_signals)
 
     def save_state(self):
-        cur_controller = self.cur_controller
-        if cur_controller is None:
+        cur = self.controllers.cur
+        if cur is None:
             return
         state_path = os.path.join(
-            self.state_dir, 'states', cur_controller._controller_name())
+            self.state_dir, 'states', cur.name)
         with open(state_path, 'w') as fp:
-            json.dump(cur_controller.serialize(), fp)
+            json.dump(cur.serialize(), fp)
 
-    def select_screen(self, index):
-        if self.cur_controller is not None:
-            self.cur_controller.end_ui()
-        self.controller_index = index
-        self.ui.progress_current = index
-        log.debug(
-            "moving to screen %s", self.cur_controller._controller_name())
-        self.cur_controller.start_ui()
+    def select_screen(self, new):
+        log.info("moving to screen %s", new.name)
+        new.start_ui()
         state_path = os.path.join(self.state_dir, 'last-screen')
         with open(state_path, 'w') as fp:
-            fp.write(self.cur_controller._controller_name())
+            fp.write(new.name)
+
+    def _move_screen(self, increment):
+        self.save_state()
+        old = self.controllers.cur
+        if old is not None:
+            old.end_ui()
+        while True:
+            self.controllers.index += increment
+            if self.controllers.out_of_bounds():
+                self.exit()
+            new = self.controllers.cur
+            try:
+                self.select_screen(new)
+            except Skip:
+                log.debug("skipping screen %s", new.name)
+                continue
+            else:
+                return
 
     def next_screen(self, *args):
-        self.save_state()
-        while True:
-            if self.controller_index == len(self.controllers) - 1:
-                self.exit()
-            try:
-                self.select_screen(self.controller_index + 1)
-            except Skip:
-                controller_name = self.controllers[self.controller_index]
-                log.debug("skipping screen %s", controller_name)
-                continue
-            else:
-                return
+        self._move_screen(1)
 
     def prev_screen(self, *args):
-        self.save_state()
-        while True:
-            if self.controller_index == 0:
-                self.exit()
-            try:
-                self.select_screen(self.controller_index - 1)
-            except Skip:
-                controller_name = self.controllers[self.controller_index]
-                log.debug("skipping screen %s", controller_name)
-                continue
-            else:
-                return
+        self._move_screen(-1)
+
+    def select_initial_screen(self, controller_index):
+        self.controllers.index = controller_index - 1
+        self.next_screen()
 
 # EventLoop -------------------------------------------------------------------
-    def redraw_screen(self):
-        if self.loop is not None:
-            try:
-                self.loop.draw_screen()
-            except AssertionError as e:
-                log.critical("Redraw screen error: {}".format(e))
 
     def exit(self):
+        state_path = os.path.join(self.state_dir, 'last-screen')
+        if os.path.exists(state_path):
+            os.unlink(state_path)
         raise urwid.ExitMainLoop()
 
     def run_scripts(self, scripts):
@@ -472,92 +568,88 @@ class Application:
 
         self.loop.set_alarm_in(0.06, _run_script)
 
+    def toggle_color(self):
+        if self.is_color:
+            new_palette = self.STYLES_MONO
+            self.is_color = False
+        else:
+            new_palette = self.color_palette
+            self.is_color = True
+        self.loop.screen.register_palette(new_palette)
+        self.loop.screen.clear()
+
     def unhandled_input(self, key):
         if key == 'ctrl x':
             self.signal.emit_signal('control-x-quit')
+        elif key == 'f3':
+            self.loop.screen.clear()
+        elif key in ['ctrl t', 'f4']:
+            self.toggle_color()
 
-    def load_controllers(self):
-        controllers_mod = __import__(
-            '{}.controllers'.format(self.project), None, None, [''])
-        for i, k in enumerate(self.controllers):
-            if self.controller_instances[k] is None:
-                log.debug("Importing controller: {}".format(k))
-                klass = getattr(controllers_mod, k+"Controller")
-                self.controller_instances[k] = klass(self)
-            else:
-                count = 1
-                for k2 in self.controllers[:i]:
-                    if k2 == k or k2.startswith(k + '-'):
-                        count += 1
-                orig = self.controller_instances[k]
-                k += '-' + str(count)
-                self.controllers[i] = k
-                self.controller_instances[k] = RepeatedController(
-                    orig, count)
-        log.debug("*** %s", self.controller_instances)
+    def start_controllers(self):
+        log.debug("starting controllers")
+        for controller in self.controllers.instances:
+            controller.start()
+        log.debug("controllers started")
 
     def load_serialized_state(self):
-        for k in self.controllers:
-            state_path = os.path.join(self.state_dir, 'states', k)
+        for controller in self.controllers.instances:
+            state_path = os.path.join(
+                self.state_dir, 'states', controller.name)
             if not os.path.exists(state_path):
                 continue
             with open(state_path) as fp:
-                self.controller_instances[k].deserialize(
-                    json.load(fp))
+                controller.deserialize(json.load(fp))
 
         last_screen = None
         state_path = os.path.join(self.state_dir, 'last-screen')
         if os.path.exists(state_path):
             with open(state_path) as fp:
                 last_screen = fp.read().strip()
-
-        if last_screen in self.controllers:
-            return self.controllers.index(last_screen)
-        else:
-            return 0
+        controller_index = 0
+        for i, controller in enumerate(self.controllers.instances):
+            if controller.name == last_screen:
+                controller_index = i
+        return controller_index
 
     def run(self):
-        if (self.opts.run_on_serial and
-                os.ttyname(0) != "/dev/ttysclp0"):
-            palette = self.STYLES_MONO
-            screen = urwid.raw_display.Screen()
-        else:
-            screen, palette = setup_screen(self.COLORS, self.STYLES)
+        log.debug("Application.run")
+        screen = make_screen(self.COLORS, self.is_linux_tty, self.opts.ascii)
 
         self.loop = urwid.MainLoop(
-            self.ui, palette=palette, screen=screen,
+            self.ui, palette=self.color_palette, screen=screen,
             handle_mouse=False, pop_ups=True,
             input_filter=self.input_filter.filter,
-            unhandled_input=self.unhandled_input)
+            unhandled_input=self.unhandled_input,
+            event_loop=AsyncioEventLoop())
+
+        if self.opts.ascii:
+            urwid.util.set_encoding('ascii')
+
+        extend_dec_special_charmap()
+
+        self.toggle_color()
 
-        log.debug("Running event loop: {}".format(
-            self.loop.event_loop))
         self.base_model = self.make_model()
         try:
             if self.opts.scripts:
                 self.run_scripts(self.opts.scripts)
 
-            self.load_controllers()
+            self.controllers.load()
 
             initial_controller_index = 0
 
             if self.updated:
                 initial_controller_index = self.load_serialized_state()
 
-            def select_initial_screen(loop, index):
-                try:
-                    self.select_screen(index)
-                except Skip:
-                    self.next_screen()
-
             self.loop.set_alarm_in(
                 0.00, lambda loop, ud: tty.setraw(0))
             self.loop.set_alarm_in(
-                0.05, select_initial_screen, initial_controller_index)
+                0.05, lambda loop, ud: self.select_initial_screen(
+                    initial_controller_index))
             self._connect_base_signals()
 
-            for k in self.controllers:
-                self.controller_instances[k].start()
+            self.start_controllers()
 
             self.loop.run()
         except Exception:
