@@ -13,83 +13,14 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import asyncio
 import crypt
 import logging
 import os
 import random
-import yaml
 import subprocess
 
 log = logging.getLogger("subiquitycore.utils")
-SYS_CLASS_NET = "/sys/class/net/"
-
-
-def environment_check(check):
-    ''' Check the environment to ensure subiquity can run without issues.
-    '''
-    log.info('Checking environment for installer requirements...')
-
-    def is_file(x):
-        return os.path.isfile(x)
-
-    def is_directory(x):
-        return os.path.isdir(x)
-
-    def is_mount(x):
-        return os.path.ismount(x)
-
-    def is_writable(x):
-        return os.access(x, os.W_OK)
-
-    def is_readable(x):
-        return os.access(x, os.R_OK)
-
-    def is_executable(x):
-        return os.access(x, os.X_OK)
-
-    check_map = {
-        'read': is_readable,
-        'write': is_writable,
-        'exec': is_executable,
-        'file': is_file,
-        'directory': is_directory,
-        'mount': is_mount,
-    }
-
-    checks = yaml.safe_load(check).get('checks', None)
-    if not checks:
-        log.error('Invalid environment check configuration')
-        return False
-
-    env_ok = True
-    for check_type in [c for c in checks
-                       if c in ['read', 'write', 'mount', 'exec']]:
-        for ftype, items in checks[check_type].items():
-            for i in items:
-                if not os.path.exists(i):
-                    if 'SNAP' in os.environ:
-                        log.warn("Adjusting path for snaps: %s",
-                                 os.environ.get('SNAP'))
-                        i = os.environ.get('SNAP') + i
-                        if not os.path.exists(i):
-                            env_ok = False
-                    else:
-                        env_ok = False
-
-                    if not env_ok:
-                        log.error('FAIL: {} is not found on the'
-                                  ' filesystem'.format(i))
-                        continue
-                if check_map[ftype](i) is False:
-                    log.error('FAIL: {} is NOT of type: {}'.format(i, ftype))
-                    env_ok = False
-                    continue
-                if check_map[check_type](i) is False:
-                    log.error('FAIL: {} does NOT have required attr:'
-                              ' {}'.format(i, check_type))
-                    env_ok = False
-
-    return env_ok
 
 
 def _clean_env(env):
@@ -131,6 +62,28 @@ def run_command(cmd, *, input=None, stdout=subprocess.PIPE,
         return cp
 
 
+async def arun_command(cmd, *, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                       encoding='utf-8', input=None, errors='replace',
+                       env=None, check=False, **kw):
+    if input is None:
+        kw['stdin'] = subprocess.DEVNULL
+    else:
+        input = input.encode(encoding)
+    log.debug("arun_command called: %s", cmd)
+    proc = await asyncio.create_subprocess_exec(
+        *cmd, stdout=stdout, stderr=stderr, env=_clean_env(env), **kw)
+    stdout, stderr = await proc.communicate(input=input)
+    if encoding:
+        stdout = stdout.decode(encoding)
+        stderr = stderr.decode(encoding)
+    log.debug("arun_command %s exited with code %s", cmd, proc.returncode)
+    if check and proc.returncode != 0:
+        raise subprocess.CalledProcessError(proc.returncode, cmd)
+    else:
+        return subprocess.CompletedProcess(
+            cmd, proc.returncode, stdout, stderr)
+
+
 def start_command(cmd, *, stdin=subprocess.DEVNULL, stdout=subprocess.PIPE,
                   stderr=subprocess.PIPE, encoding='utf-8', errors='replace',
                   env=None, **kw):
